/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001,2002 Nokia Corporation
   Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   Original Author: Guruprasad Krishnamurthy <kgprasad@hotmail.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
   $Id: request.h,v 1.9 2003/01/26 16:33:47 kds Exp $

   Contains the top level method which drives the processing
   of a valid SDP request

   Fixes:
   		Dmitry Kasatkin		: cleanup, fixes
*/
	 
#ifndef SDP_REQUEST_H
#define SDP_REQUEST_H

#include <affix/sdp.h>

void initLock(void);
void acquireReadersLock(void);
void releaseReadersLock(void);
void acquireWritersLock(void);
void releaseWritersLock(void);


int handleServiceRegistrationRequest(sdp_request_t *sdpReq, sdppdu_t *response);
int handleServiceUpdateRequest(sdp_request_t *sdpReq, sdppdu_t *response);
int handleServiceRemovalRequest(sdp_request_t *sdpReq, sdppdu_t *response);
void processRequest(sdp_request_t *request);
int handleServiceDisconnectRequest(int fd);


/*
** Given a sequence of UUIDs, find the service records
** which contain all of the UUIDs in the sequence.
** All the UUIDs must exist in some attribute of the
** service record, else it is not a match
**
** Return a GList containing the service record handles
** which matched.
*/
int compareFunction(const void * a, const void * b);


#endif //SDP_REQUEST_H
